'''
Parse dmidecode cmdline output

forked from https://github.com/huanghao/dmidecode
'''

TYPE = {
    0:  'bios',
    1:  'system',
    2:  'base board',
    3:  'chassis',
    4:  'processor',
    7:  'cache',
    8:  'port connector',
    9:  'system slot',
    10: 'on board device',
    11: 'OEM strings',
    # 13: 'bios language',
    15: 'system event log',
    16: 'physical memory array',
    17: 'memory device',
    19: 'memory array mapped address',
    24: 'hardware security',
    25: 'system power controls',
    27: 'cooling device',
    32: 'system boot',
    41: 'onboard device',
}


def parse_dmi(content):
    """
    Parse the whole dmidecode output.
    Returns a list of tuples of (type int, value dict).
    """
    info = []
    lines = iter(content.strip().splitlines())
    while True:
        try:
            line = next(lines)
        except StopIteration:
            break

        if line.startswith('Handle 0x'):
            typ = int(line.split(',', 2)[1].strip()[len('DMI type'):])
            if typ in TYPE:
                info.append((TYPE[typ], _parse_handle_section(lines)))
    return info


def _parse_handle_section(lines):
    """
    Parse a section of dmidecode output

    * 1st line contains address, type and size
    * 2nd line is title
    * line started with one tab is one option and its value
    * line started with two tabs is a member of list
    """
    data = {
        '_title': next(lines).rstrip(),
    }

    for line in lines:
        line = line.rstrip()
        if line.startswith('\t\t'):
            try:
                data[k].append(line.lstrip())
            except Exception as e:
                data[k] = []
        elif line.startswith('\t'):
            k, v = [i.strip() for i in line.lstrip().split(':', 1)]
            if v:
                data[k] = v
            else:
                data[k] = []
        else:
            break

    return data



from hwfact.parser import parse_dmi


def test_bios():
    output = """
Handle 0x0000, DMI type 0, 24 bytes
BIOS Information
	Vendor: Dell Inc.
	Version: A04
	Release Date: 09/11/2010
	Address: 0xF0000
	Runtime Size: 64 kB
	ROM Size: 8192 kB
	Characteristics:
		PCI is supported
		PNP is supported
		APM is supported
		BIOS is upgradeable
		BIOS shadowing is allowed
		ESCD support is available
		Boot from CD is supported
		Selectable boot is supported
		EDD is supported
		Japanese floppy for Toshiba 1.2 MB is supported (int 13h)
		3.5"/720 kB floppy services are supported (int 13h)
		Print screen service is supported (int 5h)
		8042 keyboard services are supported (int 9h)
		Serial services are supported (int 14h)
		Printer services are supported (int 17h)
		ACPI is supported
		USB legacy is supported
		BIOS boot specification is supported
		Function key-initiated network boot is supported
		Targeted content distribution is supported
	BIOS Revision: 0.0
"""
    assert parse_dmi(output)[0] == ('bios',  {
        '_title': "BIOS Information",
        "Vendor": "Dell Inc.",
        "Version": "A04",
        "Release Date": "09/11/2010",
        "Address": "0xF0000",
        "Runtime Size": "64 kB",
        "ROM Size": "8192 kB",
        "Characteristics": [
            "PCI is supported",
            "PNP is supported",
            "APM is supported",
            "BIOS is upgradeable",
            "BIOS shadowing is allowed",
            "ESCD support is available",
            "Boot from CD is supported",
            "Selectable boot is supported",
            "EDD is supported",
            "Japanese floppy for Toshiba 1.2 MB is supported (int 13h)",
            "3.5\"/720 kB floppy services are supported (int 13h)",
            "Print screen service is supported (int 5h)",
            "8042 keyboard services are supported (int 9h)",
            "Serial services are supported (int 14h)",
            "Printer services are supported (int 17h)",
            "ACPI is supported",
            "USB legacy is supported",
            "BIOS boot specification is supported",
            "Function key-initiated network boot is supported",
            "Targeted content distribution is supported",
        ],
        "BIOS Revision": "0.0",
    })


def test_system():
    output = """
Handle 0x0100, DMI type 1, 27 bytes
System Information
	Manufacturer: Dell Inc.
	Product Name: OptiPlex 980                 
	Version: Not Specified
	Serial Number: 35K213X
	UUID: 4C4C4544-0035-4B10-8032-B3C04F313358
	Wake-up Type: Power Switch
	SKU Number: Not Specified
	Family: Not Specified
"""
    assert parse_dmi(output)[0] == ('system', {
        "_title": "System Information",
        "Manufacturer": "Dell Inc.",
        "Product Name": "OptiPlex 980",
        "Version": "Not Specified",
        "Serial Number": "35K213X",
        "UUID": "4C4C4544-0035-4B10-8032-B3C04F313358",
        "Wake-up Type": "Power Switch",
        "SKU Number": "Not Specified",
        "Family": "Not Specified",
    })



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `hwfact` package."""


from hwfact.hwfact import size_to_gb, total_ram, to_gist


def test_size():
    hp = 'Size: 16384 MB'
    assert size_to_gb(hp) == 16
    sm = 'Size: 32 GB'
    assert size_to_gb(sm) == 32


def test_total_memory_hp():
    empty = {'Array Handle': '0x1000',
             'Asset Tag': 'Not Specified',
             'Bank Locator': 'Not Specified',
             'Configured Clock Speed': 'Unknown',
             'Data Width': '64 bits',
             'Error Information Handle': 'Not Provided',
             'Form Factor': 'DIMM',
             'Locator': 'PROC 1 DIMM 1G',
             'Manufacturer': 'Not Specified',
             'Part Number': 'Not Specified',
             'Rank': 'Unknown',
             'Serial Number': 'Not Specified',
             'Set': '1',
             'Size': 'No Module Installed',
             'Speed': 'Unknown',
             'Total Width': '72 bits',
             'Type': 'DDR3',
             'Type Detail': 'Synchronous',
             '_title': 'Memory Device'}

    memory = {'Array Handle': '0x1000',
              'Asset Tag': 'Not Specified',
              'Bank Locator': 'Not Specified',
              'Configured Clock Speed': '1066 MHz',
              'Data Width': '64 bits',
              'Error Information Handle': 'Not Provided',
              'Form Factor': 'DIMM',
              'Locator': 'PROC 1 DIMM 2D',
              'Manufacturer': 'Not Specified',
              'Part Number': 'Not Specified',
              'Rank': '2',
              'Serial Number': 'Not Specified',
              'Set': '2',
              'Size': '4096 MB',
              'Speed': '1333 MHz',
              'Total Width': '72 bits',
              'Type': 'DDR3',
              'Type Detail': 'Synchronous',
              '_title': 'Memory Device'}

    assert total_ram([empty]) == 0
    assert total_ram([memory, memory, empty]) == 8


def test_sum_up_dmidecode_HP():
    gist = to_gist(open('tests/dmiraw.txt', 'r').read())
    assert gist == {'ram_gb': 48,
                    'serial_number': 'CZXZHERO',
                    'model': 'ProLiant DL360 G7',
                    'firmware': '1.82',
                    'cpu_description': 'Intel(R) Xeon(R) CPU E5620 @ 2.40GHz',
                    'manufacturer': 'HP', 'nb_processor': 2, 'nb_thread': 16}



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    'Click>=6.0',
    # TODO: put package requirements here
]

setup_requirements = [
    'pytest-runner',
    # TODO(KhalidCK): put setup requirements (distutils extensions, etc.) here
]

test_requirements = [
    'pytest',
    # TODO: put package test requirements here
]

setup(
    name='hwfact',
    version='0.2.0',
    description="From dmicode info produce a dataset",
    long_description=readme + '\n\n' + history,
    author="Khalid Chakhmoun",
    author_email='fr.ckhalid@gmail.com',
    url='https://github.com/KhalidCK/hwfact',
    packages=find_packages(include=['hwfact']),
    include_package_data=True,
    install_requires=requirements,
    zip_safe=False,
    keywords='hwfact',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.6',
    ],
    test_suite='tests',
    tests_require=test_requirements,
    setup_requires=setup_requirements,
)



# -*- coding: utf-8 -*-

"""Main module."""

import re
import logging
from . import parser

logger = logging.getLogger(__name__)


def size_to_gb(raw):
    '''
    Convert dmidecode memory size description to GB

    Parameters
    ----------
    raw:str

    Returns
    -------
    float
        value in GB
    '''
    nb = re.search('[0-9]+', raw)
    if nb:
        nb = int(re.search('[0-9]+', raw).group())
    else:
        return 0

    if 'MB' in raw:
        return nb / 1024 if nb else 0
    elif 'GB' in raw:
        return nb
    else:
        return 0


def total_ram(memory):
    '''
    Sum all memory slot size in GB 

    Parameters
    ----------
    memory:list

    Returns
    -------
    float
    '''
    return sum([size_to_gb(elt['Size']) for elt in memory])


def to_gist(raw):
    '''
    Summarize dmidecode output

    Parameters
    ----------
    raw:str
        raw output from a parser command

    Returns
    -------
    dict
    '''
    def get_key_revision(keys):
        return [key for key in keys if 'Revision' in key][0]

    parsed = parser.parse_dmi(raw)
    bios = [section[1] for section in parsed if section[0] == 'bios'][0]
    system = [section[1] for section in parsed if section[0] == 'system'][0]
    # only non empty slot
    memory = [item[1] for item in parsed if item[0] == 'memory device']
    processor = [item[1] for item in parsed if item[0] == 'processor']

    return {'serial_number': system['Serial Number'],
            'manufacturer': system['Manufacturer'],
            'model': system['Product Name'],
            'firmware': bios[get_key_revision(bios.keys())],
            'ram_gb': total_ram(memory),
            # hypothese : all proc are the same
            'cpu_description': processor[0]['Version'],
            'nb_processor': len(processor),
            'nb_thread': sum([int(proc['Thread Count']) for proc in processor])
            }


def to_gist_batch(data):
    '''
    Parameters
    ----------
    data:dict
        look like {'hostname':raw dmidecode,...}
    Returns
    -------
    iterator
        [{'hostname':x.z,serial_number: XXX...},{'hostname':x.z.z, ...}]
    '''
    for hostname,raw in data.items():
        try:
            yield {**{'hostname':hostname},**(to_gist(raw))}
        except Exception as e:
            yield {}
            logger.info("can't parse "+hostname)
